import asyncio
import random
import sys
import traceback

from nonebot import logger
from nonebot.adapters.onebot.v11 import Bot, MessageSegment
from nonebot.adapters.onebot.v11.event import PokeNotifyEvent
from nonebot.matcher import Matcher

from amrita.plugins.chat.utils import logging
from amrita.utils.admin import send_to_admin

from ..chatmanager import FakeEvent
from ..config import config_manager
from ..event import BeforePokeEvent, PokeEvent  # 自定义事件类型
from ..matcher import MatcherManager  # 自定义匹配器
from ..utils.functions import (
    get_friend_name,
    split_message_into_chats,
)
from ..utils.libchat import get_chat, get_tokens, usage_enough
from ..utils.lock import get_group_lock, get_private_lock
from ..utils.memory import get_memory_data
from ..utils.models import InsightsModel, Message


async def poke_event(event: PokeNotifyEvent, bot: Bot, matcher: Matcher):
    """处理戳一戳事件"""

    async def handle_group_poke(event: PokeNotifyEvent, bot: Bot):
        """处理群聊中的戳一戳事件"""
        Group_Data = await get_memory_data(event=event)  # 获取群聊相关数据
        event_data = event.model_dump(exclude={"group_id"})
        event_data["group_id"] = None
        u_event = PokeNotifyEvent.model_validate(event_data)
        u_data = await get_memory_data(event=u_event)
        if config_manager.config.usage_limit.enable_usage_limit:
            if (
                data.usage >= config_manager.config.usage_limit.group_daily_limit
                and config_manager.config.usage_limit.group_daily_limit != -1
            ):
                await matcher.finish()
            elif (
                u_data.usage >= config_manager.config.usage_limit.user_daily_limit
                and config_manager.config.usage_limit.user_daily_limit != -1
            ):
                await matcher.finish()
        if not Group_Data["enable"]:  # 如果群聊功能未启用，直接返回
            return
        if not event.group_id:  # 如果群组ID不存在，直接返回
            return

        # 获取用户昵称
        user_name = (
            await bot.get_group_member_info(
                group_id=event.group_id, user_id=event.user_id
            )
        )["nickname"]

        # 构造发送的消息
        send_messages = [
            Message(role="system", content=f"{config_manager.group_train}"),
            Message(
                role="user",
                content=f"\\（戳一戳消息\\){user_name} (QQ:{event.user_id}) 戳了戳你",
            ),
        ]

        # 处理戳一戳事件并获取回复
        response = await process_poke_event(event, send_messages)
        message = (
            MessageSegment.at(user_id=event.user_id)
            + MessageSegment.text(" ")
            + MessageSegment.text(response)
        )

        # 根据配置决定消息发送方式
        if not config_manager.config.function.nature_chat_style:
            await matcher.send(message)
        else:
            await send_split_messages(response, event.user_id)

    async def handle_private_poke(event: PokeNotifyEvent, bot: Bot):
        """处理私聊中的戳一戳事件"""
        if (
            data.usage >= config_manager.config.usage_limit.user_daily_limit
            and config_manager.config.usage_limit.enable_usage_limit
            and config_manager.config.usage_limit.user_daily_limit != -1
        ):
            await matcher.finish()

        name = await get_friend_name(event.user_id, bot)  # 获取好友信息
        send_messages = [
            Message(role="system", content=f"{config_manager.group_train}"),
            Message(
                role="user",
                content=f"\\（戳一戳消息\\){name} (QQ:{event.user_id}) 戳了戳你",
            ),
        ]

        # 处理戳一戳事件并获取回复
        response = await process_poke_event(event, send_messages)
        if not config_manager.config.function.nature_chat_style:
            await matcher.send(MessageSegment.text(response))
        else:
            await send_split_messages(response, event.user_id)

    async def process_poke_event(event: PokeNotifyEvent, send_messages: list) -> str:
        """处理戳一戳事件的核心逻辑"""
        if config_manager.config.matcher_function:
            # 触发自定义事件前置处理

            poke_event = BeforePokeEvent(
                nbevent=event,
                send_message=send_messages,
                model_response="",
                user_id=event.user_id,
            )
            await MatcherManager.trigger_event(poke_event, event, bot)
            send_messages = poke_event.get_send_message().unwrap()
        # 获取聊天模型的回复
        response = await get_chat(send_messages)
        tokens = await get_tokens(
            [Message.model_validate(i) for i in send_messages], response
        )
        input_tokens = tokens.prompt_tokens
        output_tokens = tokens.completion_tokens
        insights = await InsightsModel.get()

        insights.usage_count += 1
        insights.token_output += output_tokens
        insights.token_input += input_tokens
        for d, ev in (
            (
                (data, event),
                (
                    await get_memory_data(user_id=event.user_id),
                    FakeEvent(
                        time=0,
                        self_id=0,
                        post_type="",
                        user_id=event.user_id,
                    ),
                ),
            )
            if getattr(event, "group_id", None) is not None
            else ((data, event),)
        ):
            d.usage += 1  # 增加使用次数
            d.output_token_usage += output_tokens
            d.input_token_usage += input_tokens
            await d.save(ev)
        await insights.save()

        if config_manager.config.matcher_function:
            # 触发自定义事件后置处理
            poke_event = PokeEvent(
                nbevent=event,
                send_message=send_messages,
                model_response=response.content,
                user_id=event.user_id,
            )
            await MatcherManager.trigger_event(poke_event, event, bot)
            response.content = poke_event.model_response

        # 如果开启调试模式，发送调试信息给管理员
        if logging.debug:
            await send_to_admin(f"POKEMSG {send_messages}")

        return response.content

    async def send_split_messages(response: str, user_id: int):
        """发送分段消息"""
        if response_list := split_message_into_chats(response):  # 将消息分段
            first_message = (
                MessageSegment.at(user_id) + MessageSegment.text(" ") + response_list[0]
            )
            await matcher.send(first_message)

            # 逐条发送分段消息
            for message in response_list[1:]:
                await matcher.send(message)
                await asyncio.sleep(
                    random.randint(1, 3) + len(message) // random.randint(80, 100)
                )

    async def handle_poke_exception():
        """处理戳一戳事件中的异常"""
        exc_type, exc_value, exc_traceback = sys.exc_info()
        logger.exception("发生了异常")
        logger.error(f"Exception message: {exc_value!s}")

        # 将异常信息发送给管理员
        await send_to_admin(f"出错了！{exc_value},\n{exc_type!s}")
        await send_to_admin(f"{traceback.format_exc()}")

        logger.error(
            f"Detailed exception info:\n{''.join(traceback.format_exception(exc_type, exc_value, exc_traceback))}"
        )

    # 主逻辑入口
    if (
        not config_manager.config.enable
        or not config_manager.config.function.poke_reply
    ):
        matcher.skip()  # 如果功能未启用或未配置戳一戳回复，跳过处理

    if event.target_id != event.self_id:  # 如果目标不是机器人本身，直接返回
        return
    data = await get_memory_data(event)  # 获取用户或群组相关数据
    try:
        if not await usage_enough(event) or not await usage_enough(
            FakeEvent(time=0, self_id=0, post_type="", user_id=event.user_id)
        ):  # 检查用户或群组使用次数是否超出限制
            return
        if event.group_id is not None:  # 判断是群聊还是私聊
            async with get_group_lock(event.group_id):
                await handle_group_poke(event, bot)
        else:
            async with get_private_lock(event.user_id):
                await handle_private_poke(event, bot)
    except Exception:
        await handle_poke_exception()  # 异常处理



import json
import os
import random
import typing
from collections.abc import Awaitable, Callable
from copy import deepcopy
from typing import Any

from nonebot import get_bot
from nonebot.adapters.onebot.v11 import Bot, MessageEvent
from nonebot.log import logger

from amrita.plugins.chat.utils.llm_tools.models import ToolContext
from amrita.plugins.chat.utils.logging import debug_log
from amrita.plugins.chat.utils.models import SEND_MESSAGES, ToolCall, UniResponse
from amrita.utils.admin import send_to_admin

from .config import Config, config_manager
from .event import BeforeChatEvent, ChatEvent
from .matcher import ChatException
from .on_event import on_before_chat, on_chat
from .utils.libchat import (
    tools_caller,
)
from .utils.llm_tools.builtin_tools import (
    PROCESS_MESSAGE,
    PROCESS_MESSAGE_TOOL,
    REASONING_TOOL,
    REPORT_TOOL_HIGH,
    REPORT_TOOL_LOW,
    REPORT_TOOL_MEDIUM,
    STOP_TOOL,
    report,
)
from .utils.llm_tools.manager import ToolsManager, on_tools
from .utils.memory import (
    get_memory_data,
)
from .utils.models import (
    Message,
    ToolResult,
)

prehook = on_before_chat(block=False, priority=2)
checkhook = on_before_chat(block=False, priority=1)
posthook = on_chat(block=False, priority=1)


BUILTIN_TOOLS_NAME = {
    REPORT_TOOL_MEDIUM.function.name,
    STOP_TOOL.function.name,
    REASONING_TOOL.function.name,
    PROCESS_MESSAGE.function.name,
}

AGENT_PROCESS_TOOLS = (
    REASONING_TOOL,
    STOP_TOOL,
    PROCESS_MESSAGE,
)


class Continue(BaseException): ...


@checkhook.handle()
async def text_check(event: BeforeChatEvent) -> None:
    config: Config = config_manager.config
    if not config.llm_config.tools.enable_report:
        checkhook.pass_event()
    logger.info("Content checking in progress......")
    bot = get_bot()
    match config.llm_config.tools.report_invoke_level:
        case "low":
            tool_list = [REPORT_TOOL_LOW]
        case "medium":
            tool_list = [REPORT_TOOL_MEDIUM]
        case "high":
            tool_list = [REPORT_TOOL_HIGH]
        case _:
            raise ValueError("Invalid report_invoke_level")
    msg: SEND_MESSAGES = event._send_message.unwrap()
    if (
        config.llm_config.tools.report_exclude_context
        and config.llm_config.tools.report_exclude_system_prompt
    ):
        msg = [event.get_send_message().get_user_query()]
    elif config.llm_config.tools.report_exclude_system_prompt:
        msg = event.get_send_message().get_memory()
    elif config.llm_config.tools.report_exclude_context:
        msg = [
            event.get_send_message().get_train(),
            event.get_send_message().get_user_query(),
        ]
    if not msg:
        logger.warning("Message list is empty, skipping content check")
        return
    response: UniResponse[None, list[ToolCall] | None] = await tools_caller(
        msg, tool_list
    )
    nonebot_event = typing.cast(MessageEvent, event.get_nonebot_event())
    if tool_calls := response.tool_calls:
        for tool_call in tool_calls:
            function_name = tool_call.function.name
            function_args: dict[str, Any] = json.loads(tool_call.function.arguments)
            if function_name == REPORT_TOOL_MEDIUM.function.name:
                if not function_args.get("invoke"):
                    return
                await report(
                    event,
                    function_args,
                    typing.cast(Bot, bot),
                )
                if config_manager.config.llm_config.tools.report_then_block:
                    data = await get_memory_data(nonebot_event)
                    data.memory.messages = []
                    await data.save(nonebot_event)
                    await bot.send(
                        nonebot_event,
                        random.choice(config_manager.config.llm_config.block_msg),
                    )
                    prehook.cancel_nonebot_process()
            else:
                await send_to_admin(
                    f"[LLM-Report] Detected non-passed tool call: {function_name}, please feedback this issue to the model provider."
                )


@on_tools(
    data=PROCESS_MESSAGE_TOOL,
    custom_run=True,
    show_call=False,
    enable_if=lambda: config_manager.config.llm_config.tools.agent_middle_message,
)
async def _(ctx: ToolContext) -> str | None:
    msg: str = ctx.data["content"]
    debug_log(f"[LLM-ProcessMessage] {msg}")
    await ctx.bot.send(ctx.event.get_nonebot_event(), msg)
    return f"Sent a message to user:\n\n```text\n{msg}\n```\n"


@prehook.handle()
async def agent_core(event: BeforeChatEvent) -> None:
    agent_last_step: str = ""

    async def _append_reasoning(
        msg: SEND_MESSAGES, response: UniResponse[None, list[ToolCall] | None]
    ):
        nonlocal agent_last_step
        tool_calls: list[ToolCall] | None = response.tool_calls
        if tool_calls:
            for tool in tool_calls:
                if tool.function.name == REASONING_TOOL.function.name:
                    break
            else:
                raise ValueError(f"No reasoning tool found in response \n\n{response}")
            if reasoning := json.loads(tool.function.arguments).get("content"):
                msg.append(Message.model_validate(response, from_attributes=True))
                msg.append(
                    ToolResult(
                        role="tool",
                        name=tool.function.name,
                        content=reasoning,
                        tool_call_id=tool.id,
                    )
                )
                agent_last_step = reasoning
                debug_log(f"[AmritaAgent] {reasoning}")
                if not config.llm_config.tools.agent_reasoning_hide:
                    await bot.send(nonebot_event, f"[AmritaAgent] {reasoning}")
            else:
                raise ValueError("Reasoning tool has no content!")

    async def append_reasoning_msg(
        msg: SEND_MESSAGES,
        original_msg: str = "",
        last_step: str = "",
        tools_ctx: list[dict[str, Any]] = [],
    ):
        nonlocal agent_last_step
        reasoning_msg = [
            Message(
                role="system",
                content="Please analyze the task requirements based on the user input above,"
                + " summarize the current step's purpose and reasons, and execute accordingly."
                + " If no task needs to be performed, no description is needed;"
                + " please analyze according to the character tone set in <SYS_SETTINGS> (if present)."
                + (
                    f"\nYour previous task was:\n```text\n{last_step}\n```\n"
                    if last_step
                    else ""
                )
                + (f"\n<INPUT>\n{original_msg}\n</INPUT>\n" if original_msg else "")
                + (
                    f"<SYS_SETTINGS>\n{event._send_message.train.content!s}\n</SYS_SETTINGS>"
                ),
            ),
            *deepcopy(msg),
        ]
        response: UniResponse[None, list[ToolCall] | None] = await tools_caller(
            reasoning_msg, [REASONING_TOOL.model_dump(), *tools_ctx], REASONING_TOOL
        )
        await _append_reasoning(msg, response)

    async def run_tools(
        msg_list: list,
        nonebot_event: MessageEvent,
        call_count: int = 1,
        original_msg: str = "",
    ):
        suggested_stop: bool = False

        def stop_running():
            """Mark agent workflow as completed."""
            nonlocal suggested_stop
            suggested_stop = True

        logger.debug(
            f"Starting round {call_count} tool call, current message count: {len(msg_list)}"
        )
        if config_manager.config.llm_config.tools.agent_mode_enable and (
            (
                call_count == 1
                and config_manager.config.llm_config.tools.agent_thought_mode
                == "reasoning"
            )
            or config_manager.config.llm_config.tools.agent_thought_mode
            == "reasoning-required"
        ):
            await append_reasoning_msg(msg_list, original_msg, tools_ctx=tools)

        if call_count > config_manager.config.llm_config.tools.agent_tool_call_limit:
            await bot.send(
                nonebot_event, "[AmritaAgent] 过多次的工具调用！已中止Workflow！"
            )
            msg_list.append(
                Message(
                    role="user",
                    content="Too much tools called,please call later or follow user's instruction."
                    + "Now please continue to completion.",
                )
            )
            return
        response_msg = await tools_caller(
            msg_list,
            tools,
            (
                "required"
                if (
                    config_manager.config.llm_config.tools.require_tools
                    and not suggested_stop
                )
                else "auto"
            ),
        )

        if tool_calls := response_msg.tool_calls:
            result_msg_list: list[ToolResult] = []
            for tool_call in tool_calls:
                function_name = tool_call.function.name
                function_args: dict[str, Any] = json.loads(tool_call.function.arguments)
                logger.debug(f"Function arguments are {tool_call.function.arguments}")
                logger.debug(f"Calling function {function_name}")
                try:
                    match function_name:
                        case REASONING_TOOL.function.name:
                            logger.debug("Generating task summary and reason.")
                            await _append_reasoning(msg_list, response=response_msg)
                            raise Continue()
                        case STOP_TOOL.function.name:
                            logger.debug("Agent work has been terminated.")
                            func_response = (
                                "You have indicated readiness to provide the final answer."
                                + "Please now generate the final, comprehensive response for the user."
                            )
                            if "result" in function_args:
                                logger.debug(f"[Done] {function_args['result']}")
                                func_response += (
                                    f"\nWork summary :\n{function_args['result']}"
                                )
                            msg_list.append(
                                Message.model_validate(
                                    response_msg, from_attributes=True
                                )
                            )

                            stop_running()
                        case _:
                            if (
                                tool_data := ToolsManager().get_tool(function_name)
                            ) is not None:
                                if not tool_data.custom_run:
                                    msg_list.append(
                                        Message.model_validate(
                                            response_msg, from_attributes=True
                                        )
                                    )
                                    func_response: str = await typing.cast(
                                        Callable[[dict[str, Any]], Awaitable[str]],
                                        tool_data.func,
                                    )(function_args)
                                elif (
                                    tool_response := await typing.cast(
                                        Callable[[ToolContext], Awaitable[str | None]],
                                        tool_data.func,
                                    )(
                                        ToolContext(
                                            data=function_args,
                                            event=event,
                                            matcher=prehook,
                                            bot=bot,
                                        )
                                    )
                                ) is None:
                                    func_response = "(this tool returned no content)"
                                else:
                                    msg_list.append(
                                        Message.model_validate(
                                            response_msg, from_attributes=True
                                        )
                                    )
                                    func_response = tool_response
                            else:
                                logger.opt(exception=True, colors=True).error(
                                    f"Encountered undefined function in ChatHook: {function_name}"
                                )
                                continue
                except Continue:
                    continue
                except Exception as e:
                    if isinstance(e, ChatException):
                        raise
                    logger.warning(f"Function {function_name} execution failed: {e}")
                    if (
                        config_manager.config.llm_config.tools.agent_mode_enable
                        and function_name not in BUILTIN_TOOLS_NAME
                        and config_manager.config.llm_config.tools.agent_tool_call_notice
                    ):
                        await bot.send(
                            nonebot_event, f"ERR: Tool {function_name} 执行失败"
                        )
                    msg_list.append(
                        ToolResult(
                            role="tool",
                            name=function_name,
                            content=f"ERR: Tool {function_name} execution failed\n{e!s}",
                            tool_call_id=tool_call.id,
                        )
                    )
                    continue
                else:
                    logger.debug(f"Function {function_name} returned: {func_response}")

                    msg: ToolResult = ToolResult(
                        role="tool",
                        content=func_response,
                        name=function_name,
                        tool_call_id=tool_call.id,
                    )
                    msg_list.append(msg)
                    result_msg_list.append(msg)
                finally:
                    call_count += 1
            if config_manager.config.llm_config.tools.agent_mode_enable:
                # 发送工具调用信息给用户
                if (
                    config_manager.config.llm_config.tools.agent_tool_call_notice
                    == "notify"
                ):
                    if message := (
                        "".join(
                            f"✅ 调用了工具 {i.name}\n"
                            for i in result_msg_list
                            if (
                                getattr(
                                    ToolsManager().get_tool(i.name), "on_call", None
                                )
                                == "show"
                                and i.name not in AGENT_PROCESS_TOOLS
                            )
                        )
                    ):
                        await bot.send(nonebot_event, message)

                await run_tools(msg_list, nonebot_event, call_count, original_msg)

    config = config_manager.config
    if not config.llm_config.tools.enable_tools:
        return
    nonebot_event = event.get_nonebot_event()
    if not isinstance(nonebot_event, MessageEvent):
        return
    bot = typing.cast(Bot, get_bot(str(nonebot_event.self_id)))
    msg_list: SEND_MESSAGES = (
        [
            deepcopy(event.message.train),
            deepcopy(event.message.user_query),
        ]
        if config.llm_config.tools.use_minimal_context
        else event.message.unwrap()
    )
    current_length = len(msg_list)
    chat_list_backup = event.message.copy()
    tools: list[dict[str, Any]] = []
    if config.llm_config.tools.agent_mode_enable:
        tools.append(STOP_TOOL.model_dump())
        if config.llm_config.tools.agent_thought_mode.startswith("reasoning"):
            tools.append(REASONING_TOOL.model_dump())
    tools.extend(ToolsManager().tools_meta_dict().values())
    logger.debug(
        "工具列表："
        + "".join(
            f"{tool['function']['name']}: {tool['function']['description']}\n\n"
            for tool in tools
        )
    )
    debug_log(f"工具列表：{tools}")
    if not tools:
        logger.warning("未定义任何有效工具！Tools Workflow已跳过。")
        return
    if str(os.getenv("AMRITA_IGNORE_AGENT_TOOLS")).lower() == "true" and (
        config.llm_config.tools.agent_mode_enable
        and len(tools) == len(AGENT_PROCESS_TOOLS)
    ):
        logger.warning(
            "注意：当前工具类型仅有Agent模式过程工具，而无其他有效工具定义，这通常不是使用Agent模式的最佳实践。配置环境变量AMRITA_IGNORE_AGENT_TOOLS=true可忽略此警告。"
        )

    try:
        await run_tools(
            msg_list, nonebot_event, original_msg=nonebot_event.get_plaintext()
        )
        event._send_message.memory.extend(msg_list[current_length:])

    except Exception as e:
        if isinstance(e, ChatException):
            raise
        logger.opt(colors=True, exception=e).exception(
            f"ERROR\n{e!s}\n!Failed to call Tools! Continuing with old data..."
        )
        event._send_message = chat_list_backup


@posthook.handle()
async def cookie(event: ChatEvent, bot: Bot):
    config = config_manager.config
    response = event.get_model_response()
    nonebot_event = event.get_nonebot_event()
    if config.cookies.enable_cookie:
        if cookie := config.cookies.cookie:
            if cookie in response:
                await send_to_admin(
                    f"WARNING!!!\n[{nonebot_event.get_user_id()}]{'[群' + str(getattr(nonebot_event, 'group_id', '')) + ']' if hasattr(nonebot_event, 'group_id') else ''}用户输入导致了可能的Prompt泄露！！"
                    + f"\nCookie:{cookie[:3]}......"
                    + f"\n<input>\n{nonebot_event.get_plaintext()}\n</input>\n"
                    + "输出已包含目标Cookie！已阻断消息。"
                )
                data = await get_memory_data(nonebot_event)
                data.memory.messages = []
                await data.save(nonebot_event)
                await bot.send(
                    nonebot_event,
                    random.choice(config_manager.config.llm_config.block_msg),
                )
                posthook.cancel_nonebot_process()



import asyncio
import contextlib
import copy
import random
import time
from asyncio import CancelledError, Lock, Task
from collections import defaultdict
from dataclasses import dataclass, field
from datetime import datetime
from typing import Any
from uuid import uuid4

from nonebot import logger
from nonebot.adapters.onebot.v11 import (
    Bot,
    MessageSegment,
)
from nonebot.adapters.onebot.v11.event import (
    GroupMessageEvent,
    MessageEvent,
    Reply,
)
from nonebot.matcher import Matcher
from nonebot_plugin_orm import get_session
from pydantic import BaseModel, Field
from pytz import utc
from typing_extensions import Self

from amrita.plugins.chat.matcher import ChatException
from amrita.plugins.chat.utils.event import GroupEvent
from amrita.plugins.chat.utils.logging import debug_log

from .check_rule import FakeEvent
from .config import Config, config_manager
from .event import BeforeChatEvent, ChatEvent
from .matcher import MatcherManager
from .utils.functions import (
    get_current_datetime_timestamp,
    get_friend_name,
    split_message_into_chats,
    synthesize_message,
)
from .utils.libchat import get_chat, get_tokens, text_generator
from .utils.lock import get_group_lock, get_private_lock
from .utils.memory import (
    Memory,
    MemoryModel,
    get_memory_data,
)
from .utils.models import (
    CT_MAP,
    SEND_MESSAGES,
    Content,
    ImageContent,
    ImageUrl,
    InsightsModel,
    Message,
    SessionMemoryModel,
    TextContent,
    ToolResult,
    UniResponseUsage,
)
from .utils.protocol import UniResponse
from .utils.tokenizer import hybrid_token_count

LOCK = Lock()


async def synthesize_message_to_msg(
    event: MessageEvent,
    role: str,
    date: str,
    user_name: str,
    user_id: str,
    content: str,
):
    """将消息转换为Message

    根据配置和多模态支持情况，将事件消息转换为适当的格式，
    支持文本和图片内容的组合。

    Args:
        event: 消息事件
        role: 用户角色
        date: 时间戳
        user_name: 用户名
        user_id: 用户ID
        content: 消息内容

    Returns:
        转换后的消息内容
    """
    is_multimodal: bool = (
        any(
            [
                (await config_manager.get_preset(preset=preset)).multimodal
                for preset in [
                    config_manager.config.preset,
                    *config_manager.config.preset_extension.backup_preset_list,
                ]
            ]
        )
        or len(config_manager.config.preset_extension.multi_modal_preset_list) > 0
    )

    if config_manager.config.parse_segments:
        text = (
            [
                TextContent(
                    text=f"[{role}][{date}][{user_name}（{user_id}）]说:{content}"
                )
            ]
            + [
                ImageContent(image_url=ImageUrl(url=seg.data["url"]))
                for seg in event.message
                if seg.type == "image" and seg.data.get("url")
            ]
            if is_multimodal
            else f"[{role}][{date}][{user_name}（{user_id}）]说:{content}"
        )
    else:
        text = event.message.extract_plain_text()
    return text


class ChatObjectMeta(BaseModel):
    """聊天对象元数据模型

    用于存储聊天对象的标识、事件和时间信息。
    """

    stream_id: str  # 聊天流ID
    event: MessageEvent  # 消息事件
    time: datetime = Field(default_factory=datetime.now)  # 创建时间
    last_call: datetime = Field(default_factory=datetime.now)  # 最后一次调用时间


class MemoryLimiter:
    """上下文处理器

    该类负责处理聊天上下文的记忆长度和token数量限制，通过上下文摘要和消息删除等方式，
    确保聊天上下文在预设的限制范围内，避免超出模型处理能力。
    """

    memory: MemoryModel  # 要处理的记忆模型
    config: Config  # 配置对象
    usage: UniResponseUsage | None = None  # token使用情况，初始为None
    _train: dict[str, str]  # 训练数据（系统提示词）
    _dropped_messages: list[Message[str] | ToolResult]  # 被删除的消息列表
    _copied_messages: Memory  # 原始消息副本（用于异常时回滚）
    _abstract_instruction = """<<SYS>>
你是一个专业的上下文摘要器，严格按照用户指令执行摘要任务。
<</SYS>>

<<INSTRUCTIONS>>
1. 直接摘要用户输入的内容
2. 保持原文的核心信息和关键细节
3. 不产生任何额外内容、解释或评论
4. 摘要应简洁、准确、完整
<</INSTRUCTIONS>>

<<RULE>>
- 仅对用户提供文本进行摘要
- 不添加任何解释、评论或补充信息
- 不改变原文的主要意思
- 保持客观中立的语调
<</RULE>>

<<FORMATTING>>
用户输入 → 直接输出摘要结果
<</FORMATTING>>"""

    def __init__(self, memory: MemoryModel, train: dict[str, str]) -> None:
        """初始化上下文处理器

        Args:
            memory: 要处理的记忆模型
            train: 训练数据（系统提示词）
        """
        self.memory = memory
        self.config = config_manager.config
        self._train = train

    async def __aenter__(self) -> Self:
        """异步上下文管理器入口，初始化处理状态

        Returns:
            返回自身实例以供使用
        """
        self._dropped_messages = []
        self._copied_messages = copy.deepcopy(self.memory.memory)
        debug_log(f"MemoryLimiter初始化，消息数量: {len(self.memory.memory.messages)}")
        return self

    async def _make_abstract(self):
        """生成上下文摘要

        通过调用LLM将当前记忆中的所有消息内容摘要为一段简短的内容，
        以减少上下文长度，同时保留关键信息。
        """
        debug_log("开始进行上下文摘要..")
        proportion = self.config.llm_config.memory_abstract_proportion  # 摘要比例
        dropped_part: SEND_MESSAGES = copy.deepcopy(self._dropped_messages)
        index = int(len(self.memory.memory.messages) * proportion) - len(dropped_part)
        if index < 0:
            index = 0
        idx: int | None = None
        if index:
            for idx, element in enumerate(self.memory.memory.messages):
                dropped_part.append(element)
                if (
                    getattr(element, "tool_calls", None) is not None
                ):  # 连带去除工具调用(system(tool_call),tool_call)
                    continue
                elif idx >= index:
                    break
        self.memory.memory.messages = self.memory.memory.messages[
            (idx if idx is not None else index) :  # 删除部分消息
        ]
        if dropped_part:
            msg_list: SEND_MESSAGES = [
                Message[str](role="system", content=self._abstract_instruction),
                Message[str](
                    role="user",
                    content=(
                        "消息列表：\n```text\n".join(
                            [
                                f"{it}\n"
                                for it in text_generator(
                                    self._dropped_messages + dropped_part,
                                    split_role=True,
                                )
                            ]
                        )
                        + "\n```"
                    ),
                ),
            ]
            debug_log("正在进行上下文摘要...")
            response = await get_chat(msg_list)
            usage = await get_tokens(msg_list, response)
            self.usage = usage
            debug_log(f"获取到上下文摘要：{response.content}")
            self.memory.memory.abstract = response.content
            debug_log("上下文摘要完成")
        else:
            debug_log("未进行上下文摘要")

    async def run_enforce(self):
        """执行记忆限制处理

        按顺序执行记忆长度限制和token数量限制，确保聊天上下文在预设范围内。
        该方法必须在异步上下文管理器中使用。

        Raises:
            RuntimeError: 当未在异步上下文管理器中使用时抛出
        """
        debug_log("开始执行记忆限制处理..")
        if not hasattr(self, "_dropped_messages") and not hasattr(
            self, "_copied_messages"
        ):
            raise RuntimeError(
                "MemoryLimiter is not initialized, please use use `async with MemoryLimiter(memory)` before calling."
            )
        await self._limit_length()
        await self._limit_tokens()
        if self.config.llm_config.enable_memory_abstract and self._dropped_messages:
            await self._make_abstract()
        debug_log("记忆限制处理完成")

    async def _limit_length(self):
        """控制记忆长度，删除超出限制的旧消息，移除不支持的消息。"""
        debug_log("开始执行记忆长度限制..")
        is_multimodal = (
            await config_manager.get_preset(config_manager.config.preset)
        ).multimodal
        data: MemoryModel = self.memory

        # Process multimodal messages when needed
        for message in data.memory.messages:
            if (
                isinstance(message.content, list)
                and not is_multimodal
                and message.role == "user"
            ):
                message_text = ""
                for content_part in message.content:
                    if isinstance(content_part, dict):
                        validator = CT_MAP.get(content_part["type"])
                        if not validator:
                            raise ValueError(
                                f"Invalid content type: {content_part['type']}"
                            )
                        content_part: Content = validator.model_validate(content_part)
                    if content_part["type"] == "text":
                        message_text += content_part["text"]
                message.content = message_text

        # Enforce memory length limit
        initial_count = len(data.memory.messages)
        while len(data.memory.messages) > 0:
            if data.memory.messages[0].role not in ("assistant", "user"):
                data.memory.messages.pop(0)
            elif len(data.memory.messages) > self.config.llm_config.memory_lenth_limit:
                self._dropped_messages.append(data.memory.messages.pop(0))
            else:
                break
        final_count = len(data.memory.messages)
        debug_log(f"记忆长度限制完成，删除了 {initial_count - final_count} 条消息")

    async def _limit_tokens(self):
        """控制 token 数量，删除超出限制的旧消息

        通过计算当前消息列表的token数量，当超出配置的session最大token限制时，
        逐步删除最早的消息直到满足token数量限制。
        """

        def get_token(memory: SEND_MESSAGES) -> int:
            tk_tmp: int = 0
            for msg in text_generator(memory):
                tk_tmp += hybrid_token_count(
                    msg,
                    config_manager.config.llm_config.tokens_count_mode,
                )
            return tk_tmp

        def drop_message():
            self._dropped_messages.append(data.memory.messages.pop(0))
            if data.memory.messages[0].role == "tool":
                self._dropped_messages.append(data.memory.messages.pop(0))

        train = self._train
        train_model = Message.model_validate(train)
        data = self.memory
        debug_log("开始执行token数量限制..")
        memory_l: SEND_MESSAGES = [train_model, *data.memory.messages]
        if not config_manager.config.llm_config.enable_tokens_limit:
            debug_log("token限制未启用，跳过处理")
            return
        prompt_length = hybrid_token_count(train["content"])
        if prompt_length > config_manager.config.session.session_max_tokens:
            logger.warning(
                f"提示词大小过大！为{prompt_length}>{config_manager.config.session.session_max_tokens}！请调整提示词或者设置！"
            )
            return
        tk_tmp: int = get_token(memory_l)

        initial_count = len(data.memory.messages)
        while tk_tmp > config_manager.config.session.session_max_tokens:
            if len(data.memory.messages) > 1:
                drop_message()
            else:
                break

            tk_tmp: int = get_token(memory_l)
            memory_l = [train_model, *data.memory.messages]
            await asyncio.sleep(0)  # CPU 密集型任务可能造成性能问题，我们在这里让出协程
        final_count = len(data.memory.messages)
        debug_log(f"token数量限制完成，删除了 {initial_count - final_count} 条消息")
        debug_log(f"最终token数量: {tk_tmp}")

    async def __aexit__(self, exc_type, exc_val, exc_tb) -> None:
        """异步上下文管理器出口，处理异常情况下的回滚

        当发生异常时，将消息恢复到处理前的状态，确保数据一致性。

        Args:
            exc_type: 异常类型
            exc_val: 异常值
            exc_tb: 异常追踪栈
        """
        if exc_type is not None:
            logger.warning("发生异常，正在回滚消息...")
            self.memory.memory.messages = self._copied_messages.messages
            return


class ChatObject:
    """聊天处理对象

    该类负责处理单次聊天会话，包括消息接收、上下文管理、模型调用和响应发送。
    """

    stream_id: str  # 聊天对象ID
    matcher: Matcher  # (lateinit) 匹配器
    bot: Bot  # (lateinit) Bot实例
    event: MessageEvent  # (lateinit) 消息事件
    config: Config  # 配置
    timestamp: str  # 时间戳(用于LLM)
    time: datetime  # 时间
    end_at: datetime | None = None
    data: MemoryModel  # (lateinit) 记忆文件
    train: dict[str, str]  # 训练数据（根据群聊/私聊选择）
    last_call: datetime  # 最后一次调用内部函数的时间
    _is_running: bool = False  # 是否正在运行
    _is_done: bool = False  # 是否已经完成
    _task: Task[None]
    _has_task: bool = False
    _err: BaseException | None = None
    _wait: bool = True
    _pending: bool = False  # 是否在等待队列中

    def __init__(self, run_blocking: bool = True) -> None:
        """初始化聊天对象"""
        self.timestamp = get_current_datetime_timestamp()
        self.time = datetime.now(utc)
        self.config = config_manager.config
        self.last_call = datetime.now(utc)
        self._wait = run_blocking

    def get_exception(self) -> BaseException | None:
        """
        获取任务执行过程中发生的异常

        Returns:
            BaseException | None: 如果任务执行中有异常则返回异常对象，否则返回None
        """
        return self._err

    def caller(self):
        """
        获取调用对象

        Returns:
            调用对象（通常是类的__call__方法）
        """
        return self.__call__

    def is_waitting(self) -> bool:
        """
        检查任务是否处于等待状态

        Returns:
            bool: 如果任务正在等待则返回True，否则返回False
        """
        return self._pending

    def is_running(self) -> bool:
        """
        检查任务是否正在运行

        Returns:
            bool: 如果任务正在运行则返回True，否则返回False
        """
        return self._is_running

    def is_done(self) -> bool:
        """
        检查任务是否已完成

        Returns:
            bool: 如果任务已完成则返回True，否则返回False
        """
        return self._is_done

    def terminate(self) -> None:
        """
        终止任务的执行
        设置任务状态为已完成，并取消内部任务
        """
        self._is_done = True
        self._is_running = False
        self._task.cancel()

    def __await__(self):
        if not hasattr(self, "_task"):
            raise RuntimeError("ChatObject not running")
        return self._task.__await__()

    async def __call__(self, event: MessageEvent, matcher: Matcher, bot: Bot) -> None:
        """调用聊天对象处理消息

        Args:
            event: 消息事件
            matcher: 匹配器
            bot: Bot实例
        """
        if not self._has_task:
            debug_log("正在启动聊天对象任务...")
            self._has_task = True
            self._task = asyncio.create_task(self.__call__(event, matcher, bot))
            return await self._task if self._wait else None
        if not self._is_running and not self._is_done:
            debug_log(f"开始处理聊天事件，用户ID: {event.user_id}")
            self.stream_id = uuid4().hex
            self.bot = bot
            self.matcher = matcher
            self.event = event
            self.train = copy.deepcopy(
                config_manager.group_train
                if isinstance(event, GroupMessageEvent)
                else config_manager.private_train
            )

            self._is_running = True
            try:
                lock = (
                    get_group_lock(event.group_id)
                    if isinstance(event, GroupMessageEvent)
                    else get_private_lock(event.user_id)
                )
                await chat_manager.add_chat_object(self)

                match self.config.function.chat_pending_mode:
                    case "queue":
                        debug_log("聊天队列模式")
                        self._pending = lock.locked()
                    case "single":
                        if lock.locked():
                            debug_log("聊天已被锁定，跳过")
                            return matcher.stop_propagation()
                    case "single_with_report":
                        if lock.locked():
                            debug_log("聊天已被锁定，发送报告")
                            await matcher.finish("聊天任务正在处理中，请稍后再试")

                async with lock:
                    self.last_call = datetime.now(utc)
                    self._pending = False
                    debug_log("获取锁成功，开始获取记忆数据")
                    self.data = await get_memory_data(event)
                    debug_log("记忆数据获取完成，开始运行聊天流程")

                    await self._run()
            except BaseException as e:
                if isinstance(e, ChatException):
                    raise
                debug_log(f"处理聊天事件时发生异常: {e}")
                await self._throw(e)
            finally:
                self._is_running = False
                self._is_done = True
                self._pending = False
                self.end_at = datetime.now(utc)
                chat_manager.running_chat_object_id2map.pop(self.stream_id, None)
                debug_log("聊天事件处理完成")

        else:
            raise RuntimeError(
                f"ChatObject of event <{event.model_dump_json()}> is already running or done"
            )

    async def _run(self):
        """运行聊天处理流程

        执行消息处理的主要逻辑，包括获取用户信息、处理消息内容、
        管理上下文长度和token限制，以及发送响应。
        """
        debug_log("开始运行聊天处理流程..")
        event = self.event
        data = self.data
        bot = self.bot
        user_id = event.user_id
        config = self.config
        debug_log("管理会话上下文..")
        await self._manage_sessions()
        debug_log("会话管理完成")

        if isinstance(event, GroupMessageEvent):
            # 群聊消息处理
            debug_log("处理群聊消息")
            group_id = event.group_id

            user_name = (
                (await bot.get_group_member_info(group_id=group_id, user_id=user_id))[
                    "nickname"
                ]
                if not config.function.use_user_nickname
                else event.sender.nickname
            )
            role = await self._get_user_role(group_id, user_id)
        else:
            debug_log("处理私聊消息")
            user_name = (
                await get_friend_name(event.user_id, bot=bot)
                if not isinstance(event, GroupMessageEvent)
                else event.sender.nickname
            )
            role = ""
        debug_log(f"获取用户信息完成: {user_name}, 角色: {role}")

        content = await synthesize_message(event.get_message(), bot)
        self.last_call = datetime.now(utc)
        debug_log(f"合成消息完成: {content}")

        if content.strip() == "":
            content = ""
        if event.reply:
            group_id = event.group_id if isinstance(event, GroupMessageEvent) else None
            debug_log("处理引用消息..")
            content = await self._handle_reply(event.reply, bot, group_id, content)

        reply_pics = self._get_reply_pics()
        debug_log(f"获取引用图片完成，共 {len(reply_pics)} 张")

        content = await synthesize_message_to_msg(
            event, role, self.timestamp, str(user_name), str(user_id), content
        )
        if isinstance(content, list):
            content.extend(reply_pics)
        data.memory.messages.append(Message(role="user", content=content))
        debug_log(f"添加用户消息到记忆，当前消息总数: {len(data.memory.messages)}")

        self.train["content"] = (
            "<SCHEMA>\n"
            + "你在纯文本环境工作，不允许使用MarkDown回复，你的工作环境是一个社交软件，我会提供聊天记录，你可以从这里面获取一些关键信息，比如时间与用户身份"
            + "（e.g.: [管理员/群主/自己/群员][YYYY-MM-DD weekday hh:mm:ss AM/PM][昵称（QQ号）]说:<内容>），但是请不要以聊天记录的格式做回复，而是纯文本方式。"
            + "请以你自己的角色身份参与讨论，交流时不同话题尽量不使用相似句式回复，用户与你交谈的信息在用户的消息输入内。"
            + "你的设定将在<SYSTEM_INSTRUCTIONS>标签对内，对于先前对话的摘要位于<SUMMARY>标签对内，"
            + "\n</SCHEMA>\n"
            + "<SYSTEM_INSTRUCTIONS>\n"
            + (
                self.train["content"]
                .replace("{cookie}", config.cookies.cookie)
                .replace("{self_id}", str(event.self_id))
                .replace("{user_id}", str(event.user_id))
                .replace("{user_name}", str(event.sender.nickname))
            )
            + "\n</SYSTEM_INSTRUCTIONS>"
            + f"\n<SUMMARY>\n{data.memory.abstract if config.llm_config.enable_memory_abstract else ''}\n</SUMMARY>"
        )

        debug_msg = (
            f"当前群组提示词：\n{config_manager.group_train}"
            if isinstance(event, GroupMessageEvent)
            else f"当前私聊提示词：\n{config_manager.private_train}"
        )
        debug_log(debug_msg)
        debug_log(self.train["content"])

        debug_log("开始应用记忆限制..")
        async with MemoryLimiter(self.data, self.train) as lim:
            await lim.run_enforce()
            abs_usage = lim.usage
            self.data = lim.memory
        debug_log("记忆限制应用完成")

        send_messages = self._prepare_send_messages()
        debug_log(f"准备发送消息完成，消息数量: {len(send_messages)}")
        response = await self._process_chat(send_messages, abs_usage)
        debug_log("聊天处理完成，准备发送响应")
        await self.send_response(response.content)
        debug_log("开始保存记忆数据..")
        await self.data.save(event)
        debug_log("记忆数据保存完成")

    async def send_response(self, response: str):
        """发送聊天模型的回复，根据配置选择不同的发送方式。

        Args:
            response: 模型响应内容
        """
        self.last_call = datetime.now(utc)
        debug_log(f"发送响应: {response[:50]}..")  # 只显示前50个字符
        if not self.config.function.nature_chat_style:
            await self.matcher.send(
                MessageSegment.reply(self.event.message_id)
                + MessageSegment.text(response)
            )
        elif response_list := split_message_into_chats(response):
            for message in response_list:
                await self.matcher.send(MessageSegment.text(message))
                await asyncio.sleep(
                    random.randint(1, 3) + (len(message) // random.randint(80, 100))
                )

    async def _process_chat(
        self,
        send_messages: SEND_MESSAGES,
        extra_usage: UniResponseUsage[int] | None = None,
    ) -> UniResponse[str, None]:
        """调用聊天模型生成回复，并触发相关事件。

        Args:
            send_messages: 发送消息列表
            extra_usage: 额外的token使用量信息

        Returns:
            模型响应
        """
        self.last_call = datetime.now(utc)

        def add_usage(ins: InsightsModel | MemoryModel, usage: UniResponseUsage[int]):
            if isinstance(ins, InsightsModel):
                ins.token_output += usage.completion_tokens
                ins.token_input += usage.prompt_tokens
            else:
                ins.input_token_usage += usage.prompt_tokens
                ins.output_token_usage += usage.completion_tokens

        event = self.event
        bot = self.bot
        data = self.data
        debug_log(f"开始处理聊天，发送消息数量: {len(send_messages)}")

        if config_manager.config.matcher_function:
            debug_log("触发匹配器函数..")
            chat_event = BeforeChatEvent(
                nbevent=event,
                send_message=send_messages,
                model_response="",
                user_id=event.user_id,
            )
            await MatcherManager.trigger_event(chat_event, event, bot)
            send_messages = chat_event.get_send_message().unwrap()

        debug_log("调用聊天模型..")
        response = await get_chat(send_messages)

        if config_manager.config.matcher_function:
            debug_log("触发聊天事件..")
            chat_event = ChatEvent(
                nbevent=event,
                send_message=send_messages,
                model_response=response.content or "",
                user_id=event.user_id,
            )
            await MatcherManager.trigger_event(chat_event, event, bot)
            response.content = chat_event.model_response

        debug_log("计算token使用情况..")
        tokens = await get_tokens(send_messages, response)
        # 记录模型回复
        data.memory.messages.append(
            Message[str](
                content=response.content,
                role="assistant",
            )
        )
        debug_log(f"添加助手回复到记忆，当前消息总数: {len(data.memory.messages)}")

        insights = await InsightsModel.get()
        debug_log(f"获取洞察数据完成，使用计数: {insights.usage_count}")

        # 写入全局统计
        insights.usage_count += 1
        add_usage(insights, tokens)
        if extra_usage:
            add_usage(insights, extra_usage)
        await insights.save()
        debug_log(f"更新全局统计完成，使用计数: {insights.usage_count}")

        # 写入记忆数据
        for d, ev in (
            (
                (data, event),
                (
                    await get_memory_data(user_id=event.user_id),
                    FakeEvent(
                        time=0,
                        self_id=0,
                        post_type="",
                        user_id=event.user_id,
                    ),
                ),
            )
            if hasattr(event, "group_id")
            else ((data, event),)
        ):
            d.usage += 1  # 增加使用次数
            add_usage(d, tokens)
            if extra_usage:
                add_usage(d, extra_usage)
            debug_log(f"更新记忆数据，使用次数: {d.usage}")
            await d.save(ev)

        debug_log("聊天处理完成")
        return response

    def _prepare_send_messages(
        self,
    ) -> list:
        """准备发送给聊天模型的消息列表，包括系统提示词数据和上下文。

        Returns:
            准备发送的消息列表
        """
        self.last_call = datetime.now(utc)
        debug_log("准备发送消息..")
        train: Message[str] = Message[str].model_validate(self.train)
        assert isinstance(train.content, str)
        data = self.data
        train.content += (
            f"\n以下是一些补充内容，如果与上面任何一条有冲突请忽略。\n<EXTRA>\n{data.prompt if data.prompt != '' else '无'}\n<EXTRA>"
            if self.config.function.allow_custom_prompt
            else ""
        )
        messages = [Message.model_validate(train), *copy.deepcopy(data.memory.messages)]
        debug_log(f"发送消息准备完成，共 {len(messages)} 条消息")
        return messages

    async def _handle_reply(
        self, reply: Reply, bot: Bot, group_id: int | None, content: str
    ) -> str:
        """处理引用消息：
        - 提取引用消息的内容和时间信息。
        - 格式化为可读的引用内容。

        Args:
            reply: 回复消息
            bot: Bot实例
            group_id: 群组ID（私聊为None）
            content: 原始内容

        Returns:
            格式化后的内容
        """
        self.last_call = datetime.now(utc)
        if not reply.sender.user_id:
            return content
        dt_object = datetime.fromtimestamp(reply.time)
        weekday = dt_object.strftime("%A")
        formatted_time = dt_object.strftime("%Y-%m-%d %I:%M:%S %p")
        role = (
            await self._get_user_role(group_id, reply.sender.user_id)
            if group_id
            else ""
        )

        reply_content = await synthesize_message(reply.message, bot)
        result = f"{content}\n<引用的消息>\n{formatted_time} {weekday} [{role}]{reply.sender.nickname}（QQ:{reply.sender.user_id}）说：{reply_content}\n</引用的消息>"
        debug_log(f"处理引用消息完成: {result[:50]}..")
        return result

    def _get_reply_pics(
        self,
    ) -> list[ImageContent]:
        """获取引用消息中的图片内容

        Returns:
            图片内容列表
        """
        self.last_call = datetime.now(utc)
        if reply := self.event.reply:
            msg = reply.message
            images = [
                ImageContent(image_url=ImageUrl(url=url))
                for seg in msg
                if seg.type == "image" and (url := seg.data.get("url")) is not None
            ]
            debug_log(f"获取引用图片完成，共 {len(images)} 张")
            return images
        return []

    async def _get_user_role(self, group_id: int, user_id: int) -> str:
        """获取用户在群聊中的身份（群主、管理员或普通成员）。

        Args:
            group_id: 群组ID
            user_id: 用户ID

        Returns:
            用户角色字符串
        """
        self.last_call = datetime.now(utc)
        role_data = await self.bot.get_group_member_info(
            group_id=group_id, user_id=user_id
        )
        role = role_data["role"]
        role_str = {"admin": "群管理员", "owner": "群主", "member": "普通成员"}.get(
            role, "[获取身份失败]"
        )
        debug_log(f"获取用户角色完成: {role_str}")
        return role_str

    async def _manage_sessions(
        self,
    ):
        """管理会话上下文：
        - 控制会话超时和历史记录。
        - 提供"继续"功能以恢复上下文。

        """
        self.last_call = datetime.now(utc)
        debug_log("开始管理会话上下文..")
        event = self.event
        data = self.data
        matcher = self.matcher
        bot = self.bot
        config = self.config
        if config.session.session_control:
            session_clear_map: dict[str, SessionTemp] = (
                chat_manager.session_clear_group
                if isinstance(event, GroupEvent)
                else chat_manager.session_clear_user
            )
            session_id = str(
                event.group_id
                if isinstance(event, GroupMessageEvent)
                else event.user_id
            )
            try:
                if session := session_clear_map.get(session_id):
                    debug_log(f"找到会话清除记录: {session_id}")
                    if "继续" not in event.message.extract_plain_text():
                        debug_log("消息中不包含'继续'，清除会话记录")
                        del session_clear_map[session_id]
                        return

                # 检查会话超时
                time_now = time.time()
                debug_log(
                    f"检查会话超时，当前时间: {time_now}, 数据时间戳: {data.timestamp}"
                )
                if (time_now - data.timestamp) >= (
                    float(config_manager.config.session.session_control_time * 60)
                ):
                    debug_log("会话超时，开始创建新会话..")
                    data.sessions.append(
                        SessionMemoryModel(messages=data.memory.messages, time=time_now)
                    )
                    if (
                        len(data.sessions)
                        > config_manager.config.session.session_control_history
                    ):
                        offset = (
                            len(data.sessions)
                            - config_manager.config.session.session_control_history
                        )
                        dropped_sesssions = data.sessions[:offset]
                        data.sessions = data.sessions[offset:]
                        async with get_session() as session:
                            for i in dropped_sesssions:
                                try:
                                    debug_log(f"删除过期会话: {i.id}")
                                    await i.delete(session)
                                except Exception as e:  # noqa: PERF203
                                    logger.warning(f"删除Session{i.id}失败\n{e}")
                            await session.commit()
                    data.memory.messages = []
                    timestamp = data.timestamp
                    data.timestamp = time_now
                    await data.save(event, raise_err=True)
                    if not (
                        (time_now - timestamp)
                        > float(config.session.session_control_time * 60 * 2)
                    ):
                        debug_log("发送继续聊天提示")
                        chated = await matcher.send(
                            f'如果想和我继续用之前的上下文聊天，快at我回复✨"继续"✨吧！\n（超过{config.session.session_control_time}分钟没理我我就会被系统抱走存档哦！）'
                        )
                        session_clear_map[session_id] = SessionTemp(
                            message_id=chated["message_id"], timestamp=datetime.now()
                        )

                        return await matcher.finish()

                elif (
                    session := session_clear_map.get(session_id)
                ) and "继续" in event.message.extract_plain_text():
                    debug_log("检测到'继续'消息，恢复上下文..")
                    with contextlib.suppress(Exception):
                        if time_now - session.timestamp.timestamp() < 100:
                            await bot.delete_msg(message_id=session.message_id)

                    session_clear_map.pop(session_id, None)

                    data.memory.messages = data.sessions[-1].messages
                    session = data.sessions.pop()
                    await session.delete()
                    await data.save(event, raise_err=True)
                    return await matcher.finish("让我们继续聊天吧～")

            finally:
                data.timestamp = time.time()
                debug_log("会话上下文管理完成")

    async def _throw(self, e: BaseException):
        """处理异常：
        - 通知用户出错。
        - 记录日志并通知管理员。

        Args:
            e: 异常对象
        """
        if isinstance(e, CancelledError):
            if hasattr(self, "matcher"):
                await self.matcher.send("成功终止了对话。")
            return
        self._err = e
        if hasattr(self, "matcher"):
            await self.matcher.send("出错了稍后试试吧（错误已反馈）")
        logger.opt(exception=e, colors=True).exception("程序发生了未捕获的异常")

    def get_snapshot(self) -> ChatObjectMeta:
        """获取聊天对象的快照

        Returns:
            聊天对象元数据
        """
        return ChatObjectMeta.model_validate(self, from_attributes=True)


class SessionTemp(BaseModel):
    message_id: int
    timestamp: datetime = Field(default_factory=datetime.now)


@dataclass
class ChatManager:
    session_clear_group: dict[str, SessionTemp] = field(default_factory=dict)
    session_clear_user: dict[str, SessionTemp] = field(default_factory=dict)
    custom_menu: list[dict[str, str]] = field(default_factory=list)
    running_messages_poke: dict[str, Any] = field(default_factory=dict)
    running_chat_object: defaultdict[tuple[int, bool], list[ChatObject]] = field(
        default_factory=lambda: defaultdict(list)
    )
    running_chat_object_id2map: dict[str, ChatObjectMeta] = field(default_factory=dict)

    def clean_obj(self, k: tuple[int, bool], maxitems: int = 10):
        """
        清理指定键下的运行中的聊天对象，只保留前10个对象，超出的未完成部分会被删除

        Args:
            k (tuple[int, bool]): 键值，由实例ID和是否为群聊组成
            maxitems (int, optional): 最大对象数. Defaults to 10.
        """
        objs = self.running_chat_object[k]
        if len(objs) > maxitems:
            dropped_obj = objs[maxitems:]
            objs = [obj for obj in dropped_obj if not obj.is_done()] + objs[:maxitems]
            dropped_obj = [obj for obj in dropped_obj if obj.is_done()]
            for obj in dropped_obj:
                self.running_chat_object_id2map.pop(obj.stream_id, None)
            self.running_chat_object[k] = objs

    def get_obj_key(self, event: MessageEvent) -> tuple[int, bool]:
        """
        从消息事件中提取对象键值，用于标识用户或群组的聊天会话

        Args:
            event (MessageEvent): 消息事件

        Returns:
            tuple[int, bool]: 由实例ID和是否为群聊组成的键值
        """
        is_group: bool = hasattr(event, "group_id")
        ins_id: int = getattr(event, "group_id", event.user_id)
        return (ins_id, is_group)

    def get_all_objs(self) -> list[ChatObjectMeta]:
        """
        获取所有运行中的聊天对象元数据

        Returns:
            list[ChatObjectMeta]: 所有运行中的聊天对象元数据列表
        """
        return list(self.running_chat_object_id2map.values())

    def get_objs(self, event: MessageEvent) -> list[ChatObject]:
        """
        根据消息事件获取对应的聊天对象列表

        Args:
            event (MessageEvent): 消息事件

        Returns:
            list[ChatObject]: 聊天对象列表
        """
        key: tuple[int, bool] = self.get_obj_key(event)
        return self.running_chat_object[key]

    async def clean_chat_objects(self, maxitems: int = 10) -> None:
        """
        异步清理所有运行中的聊天对象，限制每个键对应的对象数量不超过10个
        """
        async with LOCK:
            for key in self.running_chat_object.keys():
                self.clean_obj(key, maxitems)

    async def add_chat_object(self, chat_object: ChatObject) -> None:
        """
        添加新的聊天对象到运行列表中

        Args:
            chat_object (ChatObject): 聊天对象实例
        """
        async with LOCK:
            meta: ChatObjectMeta = chat_object.get_snapshot()
            self.running_chat_object_id2map[chat_object.stream_id] = meta
            key: tuple[int, bool] = self.get_obj_key(meta.event)
            self.running_chat_object[key].insert(0, chat_object)
            self.clean_obj(key, config_manager.config.function.chat_object_keep_count)


chat_manager = ChatManager()



"""聊天处理器模块"""

from nonebot import get_driver
from nonebot.adapters.onebot.v11 import (
    Bot,
)
from nonebot.adapters.onebot.v11.event import (
    MessageEvent,
)
from nonebot.matcher import Matcher

from ..chatmanager import ChatObject

command_prefix = get_driver().config.command_start or "/"


async def entry(event: MessageEvent, matcher: Matcher, bot: Bot):
    """聊天处理器入口函数

    该函数作为消息事件的入口点，处理命令前缀检查并启动聊天对象。

    Args:
        event: 消息事件
        matcher: 匹配器
        bot: Bot实例

    Returns:
        聊天处理结果
    """
    if any(
        event.message.extract_plain_text().strip().startswith(prefix)
        for prefix in command_prefix
        if prefix.strip()
    ):
        matcher.skip()
    return await (ChatObject().caller())(event, matcher, bot)



# mcp_client.py
import json
import random
from asyncio import Lock
from collections.abc import Awaitable, Callable
from contextlib import nullcontext
from copy import deepcopy
from typing import Any, overload

from fastmcp import Client
from fastmcp.client.client import CallToolResult
from fastmcp.client.transports import ClientTransportT
from mcp.types import TextContent
from nonebot import logger
from typing_extensions import Self
from zipp import Path

from amrita.plugins.chat.utils.llm_tools.manager import ToolsManager
from amrita.plugins.chat.utils.llm_tools.models import (
    FunctionDefinitionSchema,
    FunctionParametersSchema,
    MCPToolSchema,
    ToolData,
    ToolFunctionSchema,
    cast_mcp_properties_to_openai,
)

MCP_SERVER_SCRIPT_TYPE = ClientTransportT


class NOT_GIVEN:
    pass


class MCPClient:
    """可复用的MCP Client"""

    mcp_client: Client | None = None
    server_script: MCP_SERVER_SCRIPT_TYPE

    def __init__(
        self,
        server_script: MCP_SERVER_SCRIPT_TYPE,
        # headers: dict | None = None,
    ):
        self.mcp_client = None
        self.server_script: MCP_SERVER_SCRIPT_TYPE = server_script
        self.tools: list[MCPToolSchema] = []
        self.openai_tools: list[ToolFunctionSchema] = []

    async def __aenter__(self) -> Self:
        await self._connect()
        return self

    async def __aexit__(self, exc_type, exc_val, exc_tb):
        await self._close()

    async def simple_call(self, tool_name: str, data: dict[str, Any]) -> str:
        """调用 MCP 工具
        Args:
            tool_name (str): 工具名称
            data (dict[str, Any]): 工具参数
        """

        try:
            if self.mcp_client is None:
                await self._connect()
            assert self.mcp_client is not None
            response: CallToolResult = await self.mcp_client.call_tool(tool_name, data)
            ct: list[TextContent] = [
                i for i in response.content if isinstance(i, TextContent)
            ]
            return "".join([f"{i.text}\n\n" for i in ct])
        except Exception as e:
            logger.opt(exception=e, colors=True).error(
                f"Failed to call tool:{tool_name}, because {e}."
            )
            return json.dumps({"success": False, "error": str(e)})
        finally:
            await self._close()

    async def _connect(self, update_tools: bool = False):
        """连接到 MCP Server
        Args:
            update_tools (bool, optional): 是否更新工具列表。 Defaults to False.
        """
        if self.mcp_client is not None:
            raise RuntimeError("MCP Server 已经连接了！")

        server_script = self.server_script
        self.mcp_client = Client(server_script)
        await self.mcp_client.__aenter__()
        logger.info(f"✅ 成功连接到 MCP Server@{server_script}")
        if not self.tools or update_tools:
            self.tools = [
                MCPToolSchema.model_validate(i.model_dump())
                for i in await self.mcp_client.list_tools()
            ]
            logger.info(f"🛠️  可用工具: {[tool.name for tool in self.tools]}")
            self._cast_tool_to_openai()

    def _format_tools_for_openai(self):
        """将 MCP 工具格式转换为 OpenAI 工具格式"""
        openai_tools: list[ToolFunctionSchema] = [
            ToolFunctionSchema(
                strict=True,
                type="function",
                function=FunctionDefinitionSchema(
                    name=tool.name,
                    description=tool.description or f"运行名为：{tool.name}的工具",
                    parameters=FunctionParametersSchema(
                        type="object",
                        required=tool.inputSchema.required,
                        properties=cast_mcp_properties_to_openai(
                            property=tool.inputSchema.properties
                        ),
                    ),
                ),
            )
            for tool in self.tools
        ]
        return openai_tools

    def _cast_tool_to_openai(self) -> None:
        self.openai_tools = self._format_tools_for_openai()

    def get_tools(self) -> list[ToolFunctionSchema]:
        """获取 MCP 工具列表"""
        return self.openai_tools

    def get_original_tools(self) -> list[MCPToolSchema]:
        """获取原始的 MCP 工具列表"""
        return self.tools

    async def _close(self):
        """关闭连接"""
        if self.mcp_client:
            await self.mcp_client.__aexit__(None, None, None)
            self.mcp_client = None


class ClientManager:
    clients: list[MCPClient]
    script_to_clients: dict[str, MCPClient]
    name_to_clients: dict[str, MCPClient]  # 根据FunctionName映射到MCPClient
    tools_remapping: dict[
        str, str
    ]  # 针对于SuggarChat重复工具的重映射(原始名称->重映射名称)
    reversed_remappings: dict[str, str]  # 逆向映射(重映射名称->原始名称)
    _instance = None
    _lock: Lock
    _is_initialized = False  # ToolsMapping是否已经就绪

    def __new__(cls):
        if cls._instance is None:
            cls._instance = super().__new__(cls)
            cls.clients = []
            cls.name_to_clients = {}
            cls.tools_remapping = {}
            cls.reversed_remappings = {}
            cls.script_to_clients = {}
            cls._lock = Lock()
        return cls._instance

    def get_client_by_script(self, server_script: MCP_SERVER_SCRIPT_TYPE) -> MCPClient:
        """获取 MCP Client（不操作存储的MCP Server）
        Args:
            server_script (str, optional): MCP Server 脚本路径（或URI）。
        """
        return MCPClient(server_script)

    async def get_client_by_tool_name(self, tool_name: str) -> MCPClient:
        """根据工具名称获取 MCP Client
        Args:
            tool_name (str): 工具名称
        """
        async with self._lock:
            name = self.tools_remapping.get(tool_name) or tool_name
            if name in self.name_to_clients:
                return self.name_to_clients[name]
            raise RuntimeError(
                f"未找到工具：{tool_name}{f'（由`{name}`重映射）' if name != tool_name else ''}"
            )

    def _tools_wrapper(
        self, tool_name: str
    ) -> Callable[[dict[str, Any]], Awaitable[str]]:
        async def tools_runner(data: dict[str, Any]) -> str:
            client: MCPClient = await self.get_client_by_tool_name(tool_name)
            return await client.simple_call(tool_name, data)

        return tools_runner

    @overload
    def register_only(self, *, client: MCPClient) -> Self:
        """仅注册MCP Server，不进行初始化"""
        ...

    @overload
    def register_only(self, *, server_script: MCP_SERVER_SCRIPT_TYPE) -> Self:
        """仅注册MCP Server，不进行初始化"""
        ...

    def register_only(
        self,
        *,
        server_script: MCP_SERVER_SCRIPT_TYPE | None = None,
        client: MCPClient | None = None,
    ) -> Self:
        """仅注册MCP Server，不进行初始化"""
        if client is not None:
            self.clients.append(client)
        elif server_script is not None:
            client = MCPClient(server_script)
            self.clients.append(client)
        else:
            raise ValueError("请提供MCP Server脚本或MCP Client")
        return self

    @staticmethod
    async def update_tools(client: MCPClient):
        tools = client.get_tools()
        async with ClientManager._lock:
            for tool in tools:
                name = tool.function.name
                ToolsManager().remove_tool(name)
                ClientManager.name_to_clients.pop(name, None)
                if remap := ClientManager.tools_remapping.pop(name, None):
                    ClientManager.reversed_remappings.pop(remap, None)
        await ClientManager()._load_this(client)

    async def initialize_this(self, server_script: MCP_SERVER_SCRIPT_TYPE) -> Self:
        """注册并初始化单个MCP Server"""
        client: MCPClient = self.get_client_by_script(server_script)
        async with self._lock:
            try:
                await self._load_this(client)
            except Exception as e:
                logger.error(f"❌ 初始化 MCP Server@{server_script} 失败：{e}")
                raise
            else:
                self.clients.append(client)
        return self

    async def _load_this(self, client: MCPClient, fail_then_raise=True):
        try:
            tools_remapping_tmp = {}
            reversed_remappings_tmp = {}
            name_to_clients_tmp = {}
            async with client as c:
                tools = deepcopy(c.get_tools())
                for tool in tools:
                    if (
                        tool.function.name in self.tools_remapping
                        or tool.function.name in self.name_to_clients
                    ):
                        logger.warning(
                            f"{client}@{client.server_script} has a tool named {tool.function.name}, which is already registered, the old tool will be replaced."
                        )
                    name_to_clients_tmp[tool.function.name] = client
                    origin_name = tool.function.name
                    if ToolsManager().has_tool(tool.function.name):
                        remapped_name = (
                            f"referred_{random.randint(1, 100)}_{tool.function.name}"
                        )
                        logger.warning(
                            f"⚠️  工具已存在：{tool.function.name}，它将被重映射到：{remapped_name}"
                        )
                        tools_remapping_tmp[origin_name] = remapped_name
                        reversed_remappings_tmp[remapped_name] = origin_name
                        tool.function.name = remapped_name

                    ToolsManager().register_tool(
                        ToolData(
                            data=tool,
                            func=self._tools_wrapper(origin_name),
                            on_call="show",
                        )
                    )

        except Exception as e:
            if fail_then_raise:
                raise
            logger.error(f"❌ 连接到 MCP Server@{client.server_script} 失败：{e}")
        else:
            logger.info(f"✅ 加载到 MCP Server@{client.server_script} 成功")
            self.tools_remapping.update(tools_remapping_tmp)
            self.reversed_remappings.update(reversed_remappings_tmp)
            self.name_to_clients.update(name_to_clients_tmp)
            if isinstance(client.server_script, str | Path):
                server_script = str(client.server_script)
                self.script_to_clients[server_script] = client

    async def reinitalize_all(self):
        async with self._lock:
            for client in deepcopy(self.clients):
                await self.unregister_client(client.server_script, False)
                await self._load_this(client.server_script, fail_then_raise=False)

    async def initialize_all(self, lock: bool = True):
        """连接所有 MCP Server"""
        async with self._lock if lock else nullcontext():
            for client in self.clients:
                await self._load_this(client, False)
            self._is_initialized = True

    async def unregister_client(self, script_name: str | Path, lock: bool = True):
        """注销一个 MCP Server"""
        tools_manager = ToolsManager()
        async with self._lock if lock else nullcontext():
            script_name = str(script_name)
            if script_name in self.script_to_clients:
                client = self.script_to_clients.pop(script_name)
                for tool in client.openai_tools:
                    name = tool.function.name
                    tools_manager.remove_tool(name)
                    self.name_to_clients.pop(name, None)
                    if remap := self.tools_remapping.pop(name, None):
                        tools_manager.remove_tool(remap)
                        self.reversed_remappings.pop(remap, None)
                for idx, client in enumerate(self.clients):
                    if client.server_script == script_name:
                        self.clients.pop(idx)
                        break



from typing import Protocol

from nonebot import logger

debug: bool = False


class CastToStringAble(Protocol):
    def __str__(self) -> str: ...


def debug_log(msg: CastToStringAble):
    global debug
    if debug:
        logger.debug(msg)
